/*
 * File : dlReceiverBusy.C
 *
 * Description : This file contains the method descriptions for
 * the dlReceiverBusy class.
 */

#include "types.h"
#include "frame.h"
#include "dlEvents.h"
#include "dlReceiverBusy.h"
#include "dllayer.h"

// the receiverBusy variable tells us whether we should
// should go into busy state or not. It is set in the nwlayer/user
// interface. The user can toggle the value of this variable
// from TRUE to FALSE and vice-versa.

extern Boolean receiverBusy;

/*
 * Function : dlReceiverBusy()
 *
 * Description : This is the constructor. It invokes the
 *	base class constructor. Note the second argument of
 * the dlInfoXfer constructor is FALSE. This means that
 * the state variables of the data-link are not to be
 * reset.
 */
 
dlReceiverBusy::dlReceiverBusy(dllayer *dl) : dlInfoXfer(dl,FALSE)
{
}

/*
 * Function : PrintState()
 *
 * Description : Prints the state.
 */

void 
dlReceiverBusy::PrintState() 
{
	TRACE(dlSmModule,1,("State dlReceiverBusy\n"));
}


/*
 * Function : HandleIframe()
 *
 * Description : Processes a I-frame that has been received.
 *	First check for a valid N(r). If invalid, call
 *	HandleFrmrCondition(), with Z bit set.
 *	Else, send a RNR response. If receiverBusy flag has
 *	been reset to FALSE, change state to dlInfoXfer,
 *	else remain in dlReceiverBusy state.
 *  Again, note the receiverBusy flag merely allows
 *	us to simulate the receiver busy condition.
 */

dlStateMachine *
dlReceiverBusy::HandleIframe(int &errorCode)
{
	_dl->_lastFrameRecd->PrintParams();

	// Check for ack

	int nr = _dl->_lastFrameRecd->GetNr();
	int numAcked;
	if ( (numAcked = _dl->_win.ProcessAck(nr)) == -1 )
		return HandleFrmrCondition(Z,errorCode);

	SendStateResponse();

	if ( receiverBusy == TRUE )
		return this;

	// Receiver is not busy anymore ...change back to dlInfoXfer
	CHANGE_TO_INFOXFER(FALSE);
}

/*
 * Function : HandleRr()
 *
 * Description : First calls dlInfoXfer::HandleRr(). Then
 *	transmits a response if the RR received was a command
 * frame.
 */

dlStateMachine *
dlReceiverBusy::HandleRr(int &errorCode)
{
	// Call the base class method.
	dlInfoXfer::HandleRr(errorCode);

	// Send RNR response for cases that the base class
	// would not have responded.

	if (_dl->_lastFrameRecd->IsCmd() == TRUE && 
			_dl->_lastFrameRecd->GetPf() == 0)
		SendStateResponse();
	else if ( _dl->_lastFrameRecd->IsCmd() == FALSE ) {
		// Transmit a supervisory response.
		int errorCode = codeSuccessful;
		_dl->XmitFrame(errorCode,FALSE,GetSupervisoryFrame(),0,
				_dl->_win.GetVr());
		if ( errorCode != codeSuccessful)
			TRACE(dlSmModule,1,("Error sending supervisory response\n"));
	}


	if ( receiverBusy == TRUE )
		return this;

	// Receiver is not busy anymore 
	// Change back to dlInfoXfer

	CHANGE_TO_INFOXFER(FALSE);
}

/*
 * Function : GetSupervisoryFrame()
 *
 * Description : Returns RNR for the state of the DTE.
 */

int 
dlReceiverBusy::GetSupervisoryFrame()
{
	return RNR;
}

