/*
 * File : frame.C
 *
 * Description : This file contains the method definitions for the
 *	frame class.
 */

#include <stream.h>
#include <string.h>
#include "frame.h"
#include "dataPacket.h"
#include "layer.h"
#include "trace.h"

/*
 * Address bytes to be used for COMMAND and RESPONSE
 * frames. Set in main.C
 */

extern unsigned char cmdAddress;
extern unsigned char resAddress;

char * GetFrameName(int ft);

DEFINE_TRACE_MODULE(frameModule,"frameModule");

/*
 * Function : frame()
 *
 * Description : This is the default constructor. It initializes
 *	data, creates a empty dataPacket and registers with it.
 */

frame::frame() :
_data(0),
_pf(0),_ns(0),_nr(0),_formed(FALSE),_cmd(FALSE)
{
	_data = new dataPacket();
	_data->Register();
}
/*
 * Function : frame(dataPacket *aData, Boolean xmit)
 * 
 * Description : This constructor takes a dataPacket and a boolean
 *	'xmit' as arguments. The dataPacket can be some data that
 *	needs to be transmitted (xmit = TRUE) or something that has 
 *  just been received (xmit = FALSE).
 *	If data is to be transmitted, the constructor merely registers
 *	with the dataPacket. If it is a received packet - it checks
 *	the data for validity, and strips the flags, and extracts
 *	the frame parameters.
 */

frame::frame(dataPacket *aData, Boolean xmit) :
_data(aData), 
_pf(0),_ns(0),_nr(0),_formed(FALSE),_cmd(FALSE)
{
	_data->Register();
	if (xmit == FALSE ) {

		if ( CheckValid() == FALSE ) {
			_ft = INVALID; 
		}

		else {
			_ft = UNDEFINED;
			ExtractParams();
		}
	}
}

/*
 * Function : ~frame()
 *
 * Description : This is the destructor. It deregisters with
 *	the dataPacket.
 */

frame::~frame()
{
	_data->Deregister();
}

/*
 * Function : ExtractParams()
 *
 * Description : This function parses the received dataPacket
 *	and extracts all the data-link frame parameters like
 *	frame type, N(s), N(r), P/F bit etc.
 */

void
frame::ExtractParams()
{
	unsigned char 	add;
	add = *(_data->GetData()); // The address byte

	_data->PreStrip(1); // Remove the address byte

	_cntl = *(_data->GetData()); // the control byte
	_data->PreStrip(1);  // Remove the control byte
	_data->PostStrip(2); // remove the FCS

	_pf = (_cntl & PF_MASK) >> 4; 
	TRACE(frameModule,1,("Control field : %x pf : %d\n",_cntl,_pf));


	_cmd = (add == cmdAddress) ? FALSE : TRUE;

	if ( !(_cntl & MASK )) {
		// An I-frame..
		_ft = IFRAME;
		_ns = (_cntl & NS_MASK) >> 1;
		_nr = (_cntl & NR_MASK) >> 5;
		TRACE(frameModule,1,("I-FRAME recd nr: %d ns : %d\n",_nr,_ns));
		if ( _cmd != TRUE )
			_ft = UNDEFINED;
		else if (_data->Length() > MaxIframeLength ) {
			TRACE(frameModule,2,("I-frame info. field length exceeded\n"));
			_ft = IfrLengthExcess;
		}
	}
	else if (!(_cntl & ( MASK << 1))) {
		// A supervisory frame..
		_ft = _cntl & SUP_MASK;
		_nr = (_cntl & NR_MASK) >> 5;
		TRACE(frameModule,1,("Supervisory frame %x recd nr : %d\n",_ft,_nr));
		if ( _ft != RR && _ft != RNR && _ft != REJ )
			_ft = UNDEFINED;
		else
			if ( _data->Length() != 0 ) { // No data field allowed
				TRACE(frameModule,2,("SUP Frame has data field\n"));
				_ft = IncorrectLength;
			}
	}
	else {
		_ft = _cntl & UNUM_MASK; 
		TRACE(frameModule,1,("Unnumbered frame %x recd\n",_ft));

		// Check for proper command and response frames
		if ( _ft != SABM && _ft != DISC && _ft != UA && 
			 _ft != DM && _ft != FRMR )
			 _ft = UNDEFINED;
		else if ( (_ft == SABM || _ft == DISC ) && _cmd != TRUE )
				_ft = UNDEFINED;
		else if ( (_ft == UA || _ft == DM || _ft == FRMR ) && _cmd != FALSE)
				_ft = UNDEFINED;
		else {
			if ( ( _ft != FRMR && _data->Length() != 0 ) ||
				 ( _ft == FRMR && _data->Length() != 3 )) {
				TRACE(frameModule,2,("Frame with incorrect INFO field\n"));
				_ft = IncorrectLength;
			}
		}
	}
}

/*
 * Function : CheckValid()
 *
 * Description : This function checks the validity of the received
 *	data-link frame. It checks for leading and trailing flags, frame
 *	size, correct FCS sequence, and proper address field encoding.
 *	Returns TRUE if valid.
 */

Boolean
frame::CheckValid()
{
	const unsigned char *data = _data->GetData();
	int len = _data->Length();
	if ( len < 2 || *(data) != FLAG || *(data+len-1) != FLAG ) {
		TRACE(frameModule,2,("Flags are messed up\n"));
		return FALSE;
	}
	
	if ( _data->Length() < 6 ) {
		// Less than 32 bits between flags
		TRACE(frameModule,2,("Less than 32 bits between flags\n"));
		return FALSE;
	}

	unsigned short fcs;	
	if ( (fcs = CalculateCksum(FCS_INIT,data,len-1)) != FCS_RECV ) {
		TRACE(frameModule,2,("FCS : %04x messed up\n",fcs));
		return FALSE;
	}
	
	TRACE(frameModule,2,("FCS : %04x allright! \n",fcs));

	data += 1;
	if ( *data != addressA && *data != addressB ) {
		TRACE(frameModule,2,("Address byte : %x is not one of A/B\n",*data));
		return FALSE;
	}

	_data->PreStrip(1); // Remove the leading flag.
	_data->PostStrip(1); // Remove the trainling flag.

	return TRUE;
}

/*
 * Function : FormFrame()
 *
 * Description : This function forms a correct data-link frame
 *	around the dataPacket (which might be of zero length, or
 *	which might contain information). It add leading and trailing
 *	flags, control and address bytes, and FCS. If the frame
 *	is a previously formed frame, it strips it first before adding
 *	the data-link params afresh.
 */

void 
frame::FormFrame(Boolean isCmd)
{
	if ( _formed == TRUE ) {
		// It has old control byte, address byte etc. strip them off	
		_data->PreStrip(3); // flag, control byte and address
		_data->PostStrip(3); // FCS and flag
	}
	_formed = TRUE;

	_cntl = _ft | ( _pf << 4 ) | ( _ns << 1 ) | ( _nr << 5 );  
	TRACE(frameModule,1,("Control field : %d\n",_cntl));
	_cmd = isCmd;
	

	unsigned char add;
	(isCmd == TRUE ) ? 	(add = cmdAddress) : 
						(add = resAddress);
		
	_data->Prepend(&_cntl,1);
	_data->Prepend(&add,1);
	_data->Prepend(&FLAG,1);
	AppendFCS();
	_data->Append(&FLAG,1);
}

/*
 * Function : AppendFCS
 *
 * Description : This function appends a Frame Check Sequence (FCS)
 *	to the dataPacket.
 */

void
frame::AppendFCS()
{
	TRACE(frameModule,1,("Appending FCS. Packet length: %d\n",_data->Length()));
	unsigned short fcs;
	// First, calculate checksum
	fcs = CalculateCksum(FCS_INIT, _data->GetData(),_data->Length());
	
	// Now take a one's complement of the fcs and append it to frame.

	unsigned char fcsbuf[2];
	unsigned short tmp = ~fcs;
	fcsbuf[0] = tmp & 0xff;
	fcsbuf[1] = (tmp >> 8) & 0xff;
	_data->Append(fcsbuf,2);
}

/*
 * Function : PrintParams()
 *
 * Description : Utility function to print frame parameters.
 */

void 
frame::PrintParams()
{
	(_cmd == TRUE ) ? (cout << "CMD ") : (cout << "RES ");
	cout << GetFrameName(_ft);
	cout << " _nr : " << (unsigned short)_nr;
	cout << " _ns : " << (unsigned short)_ns;
	cout << " _pf : " << (unsigned short)_pf;

	if ( _ft == FRMR ) {
		_data->Print();
	}	
	cout << "\n" << flush;
}
/*
 * Function : GetFrameName()
 *
 * Description : Utility function used for debugging.
 * Given a frame type, it returns the frame type as a 
 * string.
 */

char *
GetFrameName(int ft)
{
	char frameType[10];

	switch ( ft ) {
		case IFRAME:
			strcpy(frameType,"I-frame");
			break;
		case RR:
			strcpy(frameType,"RR");
			break;
		case RNR:
			strcpy(frameType,"RNR");
			break;
		case REJ:
			strcpy(frameType,"REJ");
			break;
		case DM:
			strcpy(frameType,"DM");
			break;
		case SABM:
			strcpy(frameType,"SABM");
			break;
		case DISC:
			strcpy(frameType,"DISC");
			break;
		case UA:
			strcpy(frameType,"UA");
			break;
		case FRMR:
			strcpy(frameType,"FRMR");
			break;
		default:
			strcpy(frameType,"Unknown");
			break;
	}
	return frameType;
}

/*
 * Function : CalculateCksum()
 *
 * Description : Given a initial CRC value, a array
 * of unsigned char values (as data), and the length
 * of the data, this function returns a frame
 * check sequence for the data.
 */

unsigned short 
frame::CalculateCksum(unsigned short crc,
			const unsigned char *s, int n)
{
	unsigned int xi;

	while ( n-- ) {
		int i = (crc ^ *s++) & 0xff;

		int x4321 = (i & 0x0f);
		int x8765 = (i & 0xf0) >> 4;
		int xx = x4321 ^ x8765;
		xi = 	( xx << 12) + 
				((x4321 << 8) ^ (xx << 7)) +
				((x4321 << 3) ^ xx);
		crc = xi ^ (crc >> 8 );
	}
	return crc;
}

