/*
 * File : timerHandler.C
 *
 * Description : This file contains the method definitions
 * of the timerHandler class.
 */

#include <stdio.h>
#include "trace.h"
#include "timerHandler.h"

DEFINE_TRACE_MODULE(timerHandlerModule,"timerHandlerModule");

/*
 * Function : timerHandler
 *
 * Description : This is the construtor. It takes a timer
 * value, and sets the data members _nextto, and _xo.
 * The _nextto value is rounded off to the nearest
 * second.
 */

timerHandler::timerHandler(int timeout) : _disable(FALSE)
{
	struct timezone t;
	(void )gettimeofday(&_nextto,&t);
	if ( _nextto.tv_usec >= 500000 )
		_nextto.tv_sec++;
	_nextto.tv_sec += timeout;
	_xo = timeout;
	TRACE(timerHandlerModule,1,("sec : %d timeout : %d\n",_nextto.tv_sec,timeout));
}

/*
 * Function : ~timerHandler
 *
 * Description : Destructor. does nothing.
 */

timerHandler::~timerHandler()
{
	TRACE(timerHandlerModule,1,("timerHandler destructor\n"));
}

/*
 * Function : Print 
 *
 * Description : Diagnostic method. Prints value of _xo. 
 */

void
timerHandler::Print()
{
	TRACE(timerHandlerModule,1,("%d ",_xo));
}

/*
 * Function : EventCallback 
 *
 * Description : Function is called when timer times out.
 *	This really does nothing yet, except print out the
 *	value of _xo. Subclasses will redefine this to provide
 *	functionality to handle timeouts.
 */

void
timerHandler::EventCallback()
{
	TRACE(timerHandlerModule,1,("Entering timerHandler EventCallback\n"));
	TRACE(timerHandlerModule,1,("Val : %d\n",_xo));
	TRACE(timerHandlerModule,1,("Exiting timerHandler EventCallback\n"));
}

/*
 * Function : SetEvent
 *
 * Description : This function sets the timeout argument to 
 *	the number of seconds before this timer will timeout.
 */ 	

void 
timerHandler::SetEvent(fd_set *,
			fd_set *, fd_set *,
			struct timeval *timeout)
{
	struct timezone t;
	timeval curtime;

	/*
	 * Get time of day..
	 */
	(void)gettimeofday(&curtime,&t);
	if ( curtime.tv_usec >= 500000 )
		curtime.tv_sec++;

	/*
	 * If timer has already timed out, set timeout to 0,
	 * else to the required timeout value in seconds.
	 */

	timeout->tv_usec = 0;
	long temp = _nextto.tv_sec - curtime.tv_sec;
	(temp > 0 ) ?  (timeout->tv_sec = temp) : (timeout->tv_sec = 0);
	TRACE(timerHandlerModule,1,("temp : %d next : %d cur : %d\n",
			temp, _nextto.tv_sec, curtime.tv_sec));

	TRACE(timerHandlerModule,1,("ts : %x timeout : %d\n",this,timeout->tv_sec));
}

/*
 * Function : CheckEvent
 *
 * Description : This function checks the curtime argument to check
 * if this timer has timed out. If it has timed out, call
 * EventCallback() (if timer not disabled).
 */

void 
timerHandler::CheckEvent(fd_set *, fd_set *, fd_set *,
			struct timeval *curtime)
{
	if ( curtime->tv_sec >= _nextto.tv_sec ) {
		if ( _disable == FALSE)
			EventCallback();
		SetMarked(); // Mark it for deletion.
	}
}
/*
 * Function : Disable
 *
 * Description : Sets _disable flag to TRUE. This disables
 *	notification when the timer expires.
 */

void
timerHandler::Disable()
{
	_disable = TRUE;
}
